@@ -3,7 +3,6 @@ import numpy as np
 import pandas as pd
 import scipy
 from scipy import optimize
-from sklearn.base import BaseEstimator
 from category_encoders.ordinal import OrdinalEncoder
 import category_encoders.utils as util
 from sklearn.utils.random import check_random_state
@@ -11,7 +10,7 @@ from sklearn.utils.random import check_random_state
 __author__ = 'Jan Motl'
 
 
-class JamesSteinEncoder(BaseEstimator, util.TransformerWithTargetMixin):
+class JamesSteinEncoder(util.BaseEncoder, util.SupervisedTransformerMixin):
     """James-Stein estimator.
 
     Supported targets: binomial and continuous. For polynomial target support, see PolynomialWrapper.
@@ -136,60 +135,21 @@ class JamesSteinEncoder(BaseEstimator, util.TransformerWithTargetMixin):
     http://statweb.stanford.edu/~ckirby/brad/other/Article1977.pdf
 
     """
+    prefit_ordinal = True
+    encoding_relation = util.EncodingRelation.ONE_TO_ONE
 
-    def __init__(self, verbose=0, cols=None, drop_invariant=False, return_df=True,
-                 handle_unknown='value', handle_missing='value', model='independent', random_state=None, randomized=False, sigma=0.05):
-        self.verbose = verbose
-        self.return_df = return_df
-        self.drop_invariant = drop_invariant
-        self.drop_cols = []
-        self.cols = cols
+    def __init__(self, verbose=0, cols=None, drop_invariant=False, return_df=True, handle_unknown='value',
+                 handle_missing='value', model='independent', random_state=None, randomized=False, sigma=0.05):
+        super().__init__(verbose=verbose, cols=cols, drop_invariant=drop_invariant, return_df=return_df,
+                         handle_unknown=handle_unknown, handle_missing=handle_missing)
         self.ordinal_encoder = None
-        self._dim = None
         self.mapping = None
-        self.handle_unknown = handle_unknown
-        self.handle_missing = handle_missing
         self.random_state = random_state
         self.randomized = randomized
         self.sigma = sigma
         self.model = model
-        self.feature_names = None
 
-    # noinspection PyUnusedLocal
-    def fit(self, X, y, **kwargs):
-        """Fit encoder according to X and binary y.
-
-        Parameters
-        ----------
-
-        X : array-like, shape = [n_samples, n_features]
-            Training vectors, where n_samples is the number of samples
-            and n_features is the number of features.
-        y : array-like, shape = [n_samples]
-            Binary target values.
-
-        Returns
-        -------
-
-        self : encoder
-            Returns self.
-
-        """
-
-        # Unite parameters into pandas types
-        X, y = util.convert_inputs(X, y)
-
-        self._dim = X.shape[1]
-
-        # If columns aren't passed, just use every string column
-        if self.cols is None:
-            self.cols = util.get_obj_cols(X)
-        else:
-            self.cols = util.convert_cols_to_list(self.cols)
-
-        if self.handle_missing == 'error':
-            if X[self.cols].isnull().any().any():
-                raise ValueError('Columns to be encoded can not contain null')
+    def _fit(self, X, y, **kwargs):
 
         self.ordinal_encoder = OrdinalEncoder(
             verbose=self.verbose,
@@ -223,60 +183,7 @@ class JamesSteinEncoder(BaseEstimator, util.TransformerWithTargetMixin):
         else:
             raise ValueError("model='" + str(self.model) + "' is not a recognized option")
 
-        X_temp = self.transform(X, override_return_df=True)
-        self.feature_names = X_temp.columns.tolist()
-
-        # Store column names with approximately constant variance on the training data
-        if self.drop_invariant:
-            self.drop_cols = []
-            generated_cols = util.get_generated_cols(X, X_temp, self.cols)
-            self.drop_cols = [x for x in generated_cols if X_temp[x].var() <= 10e-5]
-            try:
-                [self.feature_names.remove(x) for x in self.drop_cols]
-            except KeyError as e:
-                if self.verbose > 0:
-                    print("Could not remove column from feature names."
-                    "Not found in generated cols.\n{}".format(e))
-        return self
-
-    def transform(self, X, y=None, override_return_df=False):
-        """Perform the transformation to new categorical data. When the data are used for model training,
-        it is important to also pass the target in order to apply leave one out.
-
-        Parameters
-        ----------
-
-        X : array-like, shape = [n_samples, n_features]
-        y : array-like, shape = [n_samples] when transform by leave one out
-            None, when transform without target information (such as transform test set)
-
-
-
-        Returns
-        -------
-
-        p : array, shape = [n_samples, n_numeric + N]
-            Transformed values with encoding applied.
-
-        """
-
-        if self.handle_missing == 'error':
-            if X[self.cols].isnull().any().any():
-                raise ValueError('Columns to be encoded can not contain null')
-
-        if self._dim is None:
-            raise ValueError('Must train encoder before it can be used to transform data.')
-
-        # Unite the input into pandas types
-        X, y = util.convert_inputs(X, y, deep=True)
-
-        # Then make sure that it is the right size
-        if X.shape[1] != self._dim:
-            raise ValueError('Unexpected input dimension %d, expected %d' % (X.shape[1], self._dim,))
-
-        if not list(self.cols):
-            return X
-
+    def _transform(self, X, y=None):
         X = self.ordinal_encoder.transform(X)
 
         if self.handle_unknown == 'error':
@@ -285,16 +192,12 @@ class JamesSteinEncoder(BaseEstimator, util.TransformerWithTargetMixin):
 
         # Loop over columns and replace nominal values with WOE
         X = self._score(X, y)
+        return X
 
-        # Postprocessing
-        # Note: We should not even convert these columns.
-        if self.drop_invariant:
-            X = X.drop(columns=self.drop_cols)
-
-        if self.return_df or override_return_df:
-            return X
-        else:
-            return X.values
+    def _more_tags(self):
+        tags = super()._more_tags()
+        tags["predict_depends_on_y"] = True
+        return tags
 
     def _train_pooled(self, X, y):
         # Implemented based on reference [1]
@@ -550,6 +453,7 @@ class JamesSteinEncoder(BaseEstimator, util.TransformerWithTargetMixin):
 
         return mapping
 
+    # todo this score function is copied 4 times
     def _score(self, X, y):
         for col in self.cols:
             # Score the column
@@ -561,19 +465,3 @@ class JamesSteinEncoder(BaseEstimator, util.TransformerWithTargetMixin):
                 X[col] = (X[col] * random_state_generator.normal(1., self.sigma, X[col].shape[0]))
 
         return X
-
-    def get_feature_names(self):
-        """
-        Returns the names of all transformed / added columns.
-
-        Returns
-        -------
-        feature_names: list
-            A list with all feature names transformed or added.
-            Note: potentially dropped features are not included!
-
-        """
-        if not isinstance(self.feature_names, list):
-            raise ValueError("Estimator has to be fitted to return feature names.")
-        else:
-            return self.feature_names
